/*
 * Copyright (c) 2003, 2004 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.template;

/**
 * The <code>TemplateEngine</code> writes its response to an object that 
 * implements this interface.
 *
 * @version $Id: Response.java,v 1.11 2004/08/28 16:20:07 hsivonen Exp $
 * @author  hsivonen
 */
public interface Response {

    /** 
     * Returns a stream to which the response data can be written. 
     *
     * @return a stream to which the response data can be written
     */
    public java.io.OutputStream getOutputStream() throws java.io.IOException;

    /** 
     * Informs the <code>Response</code> about the number of bytes that are 
     * going to be written to the stream returned by 
     * <code>getOutputStream()</code>.  The <code>TemplateEngine</code> may 
     * call this method in which case the implementing class may make some 
     * optimizations. However, the template engine does not guarantee that 
     * this method is called.
     *
     * @param length the number of bytes to be written
     */
    public void setContentLength(int length);

    /** 
     * Set the MIME content type of the response. The <code>type</code> string 
     * is suitable to be used as the value of the <code>Content-Type</code> 
     * HTTP header. The <code>TemplateEngine</code> guarantees that this 
     * method is called before writing to the stream.
     *
     * @param type the content type of the response
     */
    public void setContentType(String type);

    /** 
     * Sets of the earliest date after which the content that the response 
     * consists of is known to have been unmodified. 
     * 
     * @param date a date after which the content hasn't been modified
     */
    public void setLastModified(java.util.Date date);

    /**
     * Sets the cache invalidation keys.
     *
     * @param dependencies the dependency collector
     */
    public void setDependencies(Dependencies dependencies);
}
