/*
 * Copyright (c) 2003, 2004 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.template.test;

import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import fi.iki.hsivonen.template.Dependencies;
import fi.iki.hsivonen.template.Item;
import fi.iki.hsivonen.template.Template;
import fi.iki.hsivonen.xml.DOMUtils;

/**
 *
 * @author  hsivonen
 */
public class TestDataSource implements fi.iki.hsivonen.template.DataSource {
    
    public static final String testDoc = 
        "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n" +
        "<head><title>Testi 1</title></head>\n" +
        "<body>\n<h1>Testi 1</h1><p>Ensimm\u00E4inen kappale.</p>" +
        "<p>Toinen kappale.</p><ul><li>Item</li><li>Another</li></ul></body>\n" +
        "</html>\n";
    
    private Document doc;
    
    private Document template;
    
    /** Creates a new instance of TestDataSource */
    public TestDataSource(String templateUrl) throws Exception {
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setExpandEntityReferences(true);
        factory.setCoalescing(true);
        factory.setIgnoringComments(true);
        factory.setIgnoringElementContentWhitespace(false);
        factory.setValidating(false);
        DocumentBuilder builder = factory.newDocumentBuilder();
        this.doc = builder.parse(new InputSource(new java.io.StringReader(testDoc)));
        this.template = builder.parse(templateUrl);
    }
    
    public Template getTemplateForURL(URL context, Dependencies dependencies) {
        return new TestDocumentItem(null, null, "foo", this.template, context);
    }
    
    public Item select(URL context, Dependencies dependencies) {
        return new TestDocumentItem(null, null, "foo", this.doc, context);
    }
    
    public java.util.List select(Element query, URL context, boolean fullContent, Dependencies dependencies) {
        List rv = new LinkedList();
        DOMImplementation di = DOMUtils.newNonvalidatingDocumentBuilder().getDOMImplementation();
        for(Node n = query.getFirstChild(); n != null; n = n.getNextSibling()) {
            if(n.getNodeType() == Node.ELEMENT_NODE) {
                Document doc = di.createDocument("http://www.w3.org/1999/xhtml",  "html",  null);
                doc.replaceChild(doc.importNode(n, true),  doc.getDocumentElement());
                TestDocumentItem wrap = new TestDocumentItem(null, null, DOMUtils.textContent(DOMUtils.findElement(doc.getDocumentElement(), "http://www.w3.org/1999/xhtml", "title")), doc, context);
                rv.add(wrap);
            }
        }
        return rv;
    }    
    
    public Dependencies newDependencies() {
        return null;
    }

    
    public boolean shouldAvoidFullContent() {
        return true;
    }
    
}
